'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _HUBDUMP --> _HUBDUMP.CMD|
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012-2015 "Cluso99" (Ray Rodrick)                     |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS module uses an included module to perform most of the housekeeping.
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' Compile with PropTool + "PreSpin 0.3" by Andy Schenk www.insonix.ch/propeller                             
'' Some OS modules are derived from: 
''      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
'' See the included code for history and acknowledgments.
''
'' RR20151127   This module does......
''              Displays Hub RAM
'' RR20151219   fix lines, add usage if no parameters

'#include "__MODULE.spin"        ' include the common code for OS modules 
#include "__MODULE.spin"


VAR
  byte  tmp1[16],tmp2[16]
  long  hubaddr,lines
  

DAT
_ModuleStr    byte      "==HUBDUMP=",0        'module's name (string)
 
PRI executeCommand(stringPointer) | i
'' HUBDUMP command...

  'get the parameters
  str.stringCopy(@tmp1, str.tokenizeString(0))          ' hub address
  str.stringCopy(@tmp2, str.tokenizeString(0))          ' lines/length

  if strsize(@tmp1) == 0                                ' if no parameters, display usage
    printStringCR(string(" Dump Hub RAM..."))
    printStringCR(string("  HUBDUMP <hubaddr> <lines>"))
    return 0


  hubaddr := $0000                                      ' set defaults
  lines   := $10
  if strsize(@tmp1) > 0                                 
    hubaddr := str.hexadecimalToInteger(@tmp1)  
  if strsize(@tmp2) > 0                                 
    lines := str.hexadecimalToInteger(@tmp2)  

  repeat lines
    printstring(str.integerToHexadecimal(hubaddr, 4))
      printstring(string(": "))
    repeat i from 0 to 15
      printstring(str.integerToHexadecimal(byte[hubaddr+i], 2))
      printstring(string(" "))
    printstring(string("   "))
    repeat i from 0 to 15
      if ((byte[hubaddr+i] < $20) or (byte[hubaddr+i] > $7F))
        printstring(string("."))
      else
        printchar(byte[hubaddr+i])
    crlf    
    hubaddr+=16

  crlf    
  return 0

dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}